# zettelkasten/__main__.py


def main():
    msg = "".join(
        [
            "Folder based zettelkasten with a bibtex reference system and",
            "emacs org-mode file zettels",
        ]
    )
    print(msg)


if __name__ == "__main__":
    main()
